package com.example.demo_gcm_demo.backend;

/**
 * Created by Xing-Dong Yang on 2/11/2017.
 */

import com.google.appengine.api.datastore.DatastoreService;
import com.google.appengine.api.datastore.DatastoreServiceFactory;
import com.google.appengine.api.datastore.Entity;
import com.google.appengine.api.datastore.Key;
import com.google.appengine.api.datastore.KeyFactory;
import com.google.appengine.api.datastore.PreparedQuery;
import com.google.appengine.api.datastore.Query;
import com.google.appengine.api.datastore.Query.Filter;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.logging.Logger;


public class ContactDataStore {
    private Logger logger;
    private DatastoreService datastoreService;

    public ContactDataStore(){
        logger = Logger.getLogger(Contact.class.getName());
        datastoreService = DatastoreServiceFactory.getDatastoreService();
    }

    public Key getParentKey(){
        Key retKey = KeyFactory.createKey(Contact.PARENT_KIND, Contact.PARENT_IDENTIFIER);
        return retKey;
    }

    public Contact getContactByIdentifier(String name){
        Entity entity = null;
        Key key = KeyFactory.createKey(getParentKey(), Contact.CONTACT_ENTITY_KIND, name);
        try {
            entity = datastoreService.get(key);
        }catch (Exception e){}
        if(entity != null)
            return convertEntity2Contact(entity);
        else
            return null;
    }

    private Contact convertEntity2Contact(Entity entity){
        Contact ret = new Contact((String)entity.getProperty(Contact.PROPERTY_NAME),
                (String)entity.getProperty(Contact.PROPERTY_ADDRESS),
                (String)entity.getProperty(Contact.PROPERTY_PHONENUMBER));
        return ret;
    }

    public boolean addContact2Datastore(Contact contact){
        if(getContactByIdentifier(contact.name) != null)
            return false;
        else{
            Entity entity = new Entity(Contact.CONTACT_ENTITY_KIND, contact.name, getParentKey());
            entity.setProperty(Contact.PROPERTY_NAME, contact.name);
            entity.setProperty(Contact.PROPERTY_ADDRESS, contact.address);
            entity.setProperty(Contact.PROPERTY_PHONENUMBER, contact.phoneNumber);
            datastoreService.put(entity);
            return true;
        }
    }

    public boolean deleteContact(String name){
        Filter filter = new Query.FilterPredicate(Contact.PROPERTY_NAME, Query.FilterOperator.EQUAL, name);
        Query query = new Query(Contact.CONTACT_ENTITY_KIND);
        query.setFilter(filter);
        query.setAncestor(getParentKey());
        PreparedQuery preparedQuery = datastoreService.prepare(query);
        Entity entity = preparedQuery.asSingleEntity();
        if(entity == null)
            return false;
        else{
            datastoreService.delete(entity.getKey());
            return true;
        }
    }

    public ArrayList<Contact> query(String name){
        ArrayList<Contact> ret = new ArrayList<>();
        if(name == null || name.equals("")) {
            Query query = new Query(Contact.CONTACT_ENTITY_KIND);
            query.setFilter(null);
            query.setAncestor(getParentKey());
            PreparedQuery preparedQuery = datastoreService.prepare(query);
            Iterator<Entity> entities = preparedQuery.asIterator();
            while (entities.hasNext()) {
                Entity temp = entities.next();
                if (temp != null)
                    ret.add(convertEntity2Contact(temp));
            }
        }else{
            Contact contact = getContactByIdentifier(name);
            if(contact != null)
                ret.add(contact);
        }
        return ret;
    }
}
